@@ -8,13 +8,15 @@ from Cryptodome.Random import get_random_bytes
 from Cryptodome.Util.strxor import strxor
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives.asymmetric import ec
+
 # from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
 from cryptography.hazmat.primitives.ciphers.aead import AESGCM
 from cryptography.hazmat.primitives.serialization import Encoding, PublicFormat
 
 from .errors import CryptError
-from .keys import HMAC_KEY
-from .util import from_b, log_bytes, timeit, to_b
+from .keys import HMAC_KEY_PS4, HMAC_KEY_PS5
+from .util import from_b, log_bytes, to_b
+from .const import TYPE_PS5
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -22,60 +24,71 @@ GMAC_REFRESH_IV = 44910
 GMAC_REFRESH_KEY_POS = 45000
 
 
-def get_gmac_key(gmac_index: int, key: bytes, iv: bytes) -> bytes:
+def get_gmac_key(gmac_index: int, key: bytes, init_vector: bytes) -> bytes:
     """Return GMAC key."""
     gmac_index *= GMAC_REFRESH_IV
-    out_array = counter_add(gmac_index, iv)
-    out_key = b''.join([key, out_array])
+    out_array = counter_add(gmac_index, init_vector)
+    out_key = b"".join([key, out_array])
     gcm = SHA256.new(out_key)
     gcm = gcm.digest()
     gcm = strxor(gcm[0:16], gcm[16:])
     return gcm
 
 
-def counter_add(counter: int, iv: bytes) -> bytes:
-    """Increment IV by counter."""
-    # LE to BE -> Add counter -> BE to LE
-    # iv = bytes(bytearray(to_b(from_b(bytes(bytearray(iv)[::-1])) + counter, 16))[::-1])
-    iv = bytearray(iv)
-    for index, value in enumerate(iv):
+def counter_add(counter: int, init_vector: bytes) -> bytes:
+    """Increment IV by counter.
+    Operations:
+    - Convert IV from LE to BE -> Add counter -> convert from BE to LE
+    """
+    # init_vector = bytes(
+    #     bytearray(to_b(from_b(bytes(bytearray(init_vector)[::-1])) + counter, 16))[::-1]
+    # )
+    init_vector = bytearray(init_vector)
+    for index, value in enumerate(init_vector):
         add = value + counter
-        iv[index] = add & 0xff
+        init_vector[index] = add & 0xFF
         if (counter := add >> 8) <= 0 or index >= 15:
             break
-    return bytes(iv)
+    return bytes(init_vector)
 
 
 def get_base_key_iv(secret: bytes, handshake_key: bytes, index: int) -> bytes:
-    key_iv = b''.join([
-        bytes([0x01, index, 0x00]), handshake_key, bytes([0x01, 0x00])
-    ])
+    """Return Base Key and IV.
+
+    Operations:
+    - Concat index and handshake key
+    - Get HMAC hash of the above using the secret as the key
+    - Split hash into key and iv
+    """
+    key_iv = b"".join([bytes([0x01, index, 0x00]), handshake_key, bytes([0x01, 0x00])])
     hmac = HMAC.new(key=secret, msg=key_iv, digestmod=SHA256)
     hmac = hmac.digest()
     hmac = bytearray(hmac)
     key = bytes(hmac[0:16])
-    iv = bytes(hmac[16:])
-    return key, iv
+    init_vector = bytes(hmac[16:])
+    return key, init_vector
 
 
-def get_gmac_cipher(key: bytes, iv: bytes) -> bytes:
+def get_gmac_cipher(key: bytes, init_vector: bytes) -> bytes:
     """Return GMAC Cipher."""
-    cipher = AES.new(key, AES.MODE_GCM, nonce=iv, mac_len=4)
+    cipher = AES.new(key, AES.MODE_GCM, nonce=init_vector, mac_len=4)
     return cipher
 
 
-def get_gmac_tag(
-        data: bytes, key: bytes, iv: bytes) -> bytes:
+def get_gmac_tag(data: bytes, key: bytes, init_vector: bytes) -> bytes:
     """Return GMAC tag of packet."""
     cipher = AESGCM(key)
-    return cipher.encrypt(iv, b'', data)[:4]
-    # cipher = get_gmac_cipher(key, iv)
+    return cipher.encrypt(init_vector, b"", data)[:4]
+    # cipher = get_gmac_cipher(key, init_vector)
     # cipher.update(data)
     # return cipher.digest()
 
 
-def gen_iv_stream(buf: bytearray, iv: bytes, key_pos: int):
-    """Pack buf with stream of incremented IVs."""
+def gen_iv_stream(buf: bytearray, init_vector: bytes, key_pos: int):
+    """Pack buf with stream of incremented IVs.
+
+    This concats the next block of IVs needed like AES CTR.
+    """
     length = len(buf)
     assert length % 16 == 0
     blocks = length // 16
@@ -83,13 +96,14 @@ def gen_iv_stream(buf: bytearray, iv: bytes, key_pos: int):
     stop = blocks + block_offset
     current = 0
     for block in range(block_offset, stop):
-        pack_into("!16s", buf, current, counter_add(block, iv))
+        pack_into("!16s", buf, current, counter_add(block, init_vector))
         current += 16
 
 
 # TODO: Make more efficient
 def get_key_stream(
-        key: bytes, iv: bytes, key_pos: int, data_len: int) -> bytes:
+    key: bytes, init_vector: bytes, key_pos: int, data_len: int
+) -> bytes:
     """Return the minimum CTR Keystream at key position."""
     padding = key_pos % 16
     key_pos = key_pos - padding
@@ -97,7 +111,7 @@ def get_key_stream(
     key_stream_len = ((padding + data_len + 16 - 1) // 16) * 16
 
     key_stream = bytearray(key_stream_len)
-    gen_iv_stream(key_stream, iv, key_pos)
+    gen_iv_stream(key_stream, init_vector, key_pos)
 
     # cipher = Cipher(algorithms.AES(key), modes.ECB())
     # encryptor = cipher.encryptor()
@@ -113,16 +127,22 @@ def get_key_stream(
     return key_stream
 
 
-def decrypt_encrypt(key: bytes, iv: bytes, key_pos: int, data: bytes, key_stream=b''):
-    """Return Decrypted or Encrypted packet. Two way."""
+def decrypt_encrypt(
+    key: bytes, init_vector: bytes, key_pos: int, data: bytes, key_stream=b""
+):
+    """Return Decrypted or Encrypted packet. Two way. Essentially AES ECB."""
     if not key_stream:
-        key_stream = get_key_stream(key, iv, key_pos, len(data))
+        key_stream = get_key_stream(key, init_vector, key_pos, len(data))
     enc_data = strxor(data, key_stream)
     return enc_data
 
 
-class BaseCipher():
-    """Base AES CTR Cipher."""
+class BaseCipher:
+    """Base AES CTR Cipher.
+
+    Allows for random access.
+    """
+
     KEYSTREAM_LEN = 0x1000
 
     def __init__(self, handshake_key, secret):
@@ -139,18 +159,24 @@ class BaseCipher():
 
     def _init_cipher(self):
         self.base_key, self.base_iv = get_base_key_iv(
-            self.secret, self.handshake_key, self._base_index)
-        self.current_key = self.base_gmac_key = get_gmac_key(self.index, self.base_key, self.base_iv)
+            self.secret, self.handshake_key, self._base_index
+        )
+        self.current_key = self.base_gmac_key = get_gmac_key(
+            self.index, self.base_key, self.base_iv
+        )
         self._next_key_stream()
 
     def _next_key_stream(self):
         while len(self.keystreams) < 3:
             key_pos = self.keystream_index * BaseCipher.KEYSTREAM_LEN
-            key_stream = get_key_stream(self.base_key, self.base_iv, key_pos, BaseCipher.KEYSTREAM_LEN)
+            key_stream = get_key_stream(
+                self.base_key, self.base_iv, key_pos, BaseCipher.KEYSTREAM_LEN
+            )
             self.keystreams.append((self.keystream_index, key_stream))
             self.keystream_index += 1
 
     def get_key_stream(self, key_pos: int, data_len: int) -> bytes:
+        """Return required key stream."""
         self._next_key_stream()
         # Remove block if key pos not in queue.
         for index, key_stream in enumerate(self.keystreams):
@@ -159,7 +185,7 @@ class BaseCipher():
                 self.keystreams.pop(index)
             else:
                 break
-        key_stream = b''
+        key_stream = b""
         if self.keystreams:
             requires_additional = False
             start_pos = key_pos % BaseCipher.KEYSTREAM_LEN
@@ -177,16 +203,16 @@ class BaseCipher():
         return key_stream
 
     def gen_new_key(self):
+        """Generate new GMAC Key."""
         if self.base_gmac_key is None:
             raise CryptError("Base GMAC Key is None")
-        self.current_key = get_gmac_key(
-            self.index, self.base_gmac_key, self.base_iv)
+        self.current_key = get_gmac_key(self.index, self.base_gmac_key, self.base_iv)
         # _LOGGER.debug("Cipher: %s, Index: %s", self.name, self.index)
         return self.current_key
 
     def get_gmac(self, data: bytes, key_pos: int):
         """Get GMAC tag of packet."""
-        iv = counter_add(key_pos // 16, self.base_iv)
+        init_vector = counter_add(key_pos // 16, self.base_iv)
         if key_pos > 0:
             index = (key_pos - 1) // GMAC_REFRESH_KEY_POS
         else:
@@ -198,7 +224,7 @@ class BaseCipher():
             key = get_gmac_key(index, self.base_key, self.base_iv)
         else:
             key = self.current_key
-        tag = get_gmac_tag(data, key, iv)
+        tag = get_gmac_tag(data, key, init_vector)
         return tag
 
 
@@ -213,7 +239,7 @@ class RemoteCipher(BaseCipher):
 
     def decrypt(self, data: bytes, key_pos: int) -> bytes:
         """Decrypt data."""
-        key_stream = b''
+        key_stream = b""
         key_stream = self.get_key_stream(key_pos, len(data))
         dec = decrypt_encrypt(self.base_key, self.base_iv, key_pos, data, key_stream)
         return dec
@@ -238,23 +264,24 @@ class LocalCipher(BaseCipher):
         self._key_pos = 0
         self._init_cipher()
 
-    def get_gmac(self, data: bytes) -> bytes:
+    def get_gmac(self, data: bytes) -> bytes:  # pylint: disable=arguments-differ
         """Return GMAC Tag."""
         tag = super().get_gmac(data, self.key_pos)
         return tag
 
     def encrypt(self, data: bytes) -> bytes:
         """Encrypt data using key stream."""
-        key_stream = b''
+        key_stream = b""
         key_stream = self.get_key_stream(self.key_pos, len(data))
-        enc = decrypt_encrypt(self.base_key, self.base_iv, self.key_pos, data, key_stream)
+        enc = decrypt_encrypt(
+            self.base_key, self.base_iv, self.key_pos, data, key_stream
+        )
         return enc
 
     def advance_key_pos(self, advance_by: int):
         """Advance key pos by data length."""
         self._key_pos += advance_by
-        _LOGGER.debug(
-            "Advancing key pos by %s to: %s", advance_by, self.key_pos)
+        _LOGGER.debug("Advancing key pos by %s to: %s", advance_by, self.key_pos)
 
     @property
     def key_pos(self) -> int:
@@ -262,7 +289,7 @@ class LocalCipher(BaseCipher):
         return self._key_pos
 
 
-class StreamCipher():
+class StreamCipher:
     """Collection of Local and Remote Ciphers."""
 
     def __init__(self, handshake_key, secret):
@@ -297,80 +324,87 @@ class StreamCipher():
         return self._local_cipher.key_pos
 
 
-def get_aes_cipher(key: bytes, iv: bytes, segment_size=128):
-    """Get AES Cipher."""
-    # Segment size is in bits. AES-CFB-128.
-    cipher = AES.new(
-        key, AES.MODE_CFB, iv, segment_size=segment_size)
+def get_aes_cipher(key: bytes, init_vector: bytes, segment_size=128):
+    """Get AES Cipher.
+
+    Segment size is in bits. AES-CFB-128.
+    """
+    cipher = AES.new(key, AES.MODE_CFB, init_vector, segment_size=segment_size)
     return cipher
 
 
-def get_hmac(nonce: bytes):
+def get_hmac(host_type: str, nonce: bytes) -> bytes:
     """Return HMAC for the IV."""
-    hmac = HMAC.new(key=HMAC_KEY, msg=nonce, digestmod=SHA256)
+    key = HMAC_KEY_PS5 if host_type.upper() == TYPE_PS5 else HMAC_KEY_PS4
+    hmac = HMAC.new(key=key, msg=nonce, digestmod=SHA256)
     current_hmac = hmac.digest()
     # log_bytes('Current HMAC', current_hmac)
     return current_hmac
 
 
-def get_aes_iv(nonce: bytes, counter: int):
+def get_aes_iv(host_type: str, nonce: bytes, counter: int):
     """Get IV for AES Cipher as the truncated HMAC."""
     # _LOGGER.debug("IV Counter: %s", counter)
     shift = 56
     suffix = bytearray(8)
     for index in range(0, 8):
-        suffix[index] = ((counter >> shift) & 0xff)
+        suffix[index] = (counter >> shift) & 0xFF
         shift -= 8
-    nonce = b''.join([nonce, bytes(suffix)])
-    current_hmac = get_hmac(bytes(nonce))
-    iv = current_hmac[:16]  # Truncate to IV length.
-    return iv
+    nonce = b"".join([nonce, bytes(suffix)])
+    current_hmac = get_hmac(host_type, bytes(nonce))
+    init_vector = current_hmac[:16]  # Truncate to IV length.
+    return init_vector
 
 
-def get_ciphers(key: bytes, nonce: bytes, counter=0) -> tuple:
+def get_ciphers(host_type: str, key: bytes, nonce: bytes, counter=0) -> tuple:
     """Return tuple of AES CFB Ciphers."""
-    iv = get_aes_iv(nonce, counter=counter)
-    enc_cipher = get_aes_cipher(key=key, iv=iv)
-    dec_cipher = get_aes_cipher(key=key, iv=iv)
+    init_vector = get_aes_iv(host_type, nonce, counter=counter)
+    enc_cipher = get_aes_cipher(key=key, init_vector=init_vector)
+    dec_cipher = get_aes_cipher(key=key, init_vector=init_vector)
     return enc_cipher, dec_cipher
 
 
-def get_cipher(key: bytes, nonce: bytes, counter=0):
+def get_cipher(host_type: str, key: bytes, nonce: bytes, counter=0):
     """Return a AES CFB Cipher."""
-    iv = get_aes_iv(nonce, counter=counter)
-    cipher = get_aes_cipher(key=key, iv=iv)
+    init_vector = get_aes_iv(host_type, nonce, counter=counter)
+    cipher = get_aes_cipher(key=key, init_vector=init_vector)
     return cipher
 
 
-class SessionCipher():
+class SessionCipher:
     """AES CFB-128 Cipher pair."""
 
-    def __init__(self, key: bytes, nonce: bytes, counter=0):
+    def __init__(self, host_type: str, key: bytes, nonce: bytes, counter=0):
+        self._host_type = host_type
         self._enc_cipher = None
         self._dec_cipher = None
         self._key = key
         self._nonce = nonce
         self._enc_counter = self._dec_counter = counter
 
-        self._enc_cipher, self._dec_cipher = get_ciphers(key, nonce, counter)
+        self._enc_cipher, self._dec_cipher = get_ciphers(host_type, key, nonce, counter)
 
     def encrypt(self, msg: bytes, counter=None) -> bytes:
         """Return Encrypted Message."""
         if counter is not None:
-            _enc_cipher = get_cipher(self._key, self._nonce, counter)
+            _enc_cipher = get_cipher(self._host_type, self._key, self._nonce, counter)
             enc = _enc_cipher.encrypt(msg)
             return enc
 
         enc = self._enc_cipher.encrypt(msg)
         self._enc_counter += 1
-        self._enc_cipher = get_cipher(self._key, self._nonce, self.enc_counter)
+        self._enc_cipher = get_cipher(
+            self._host_type, self._key, self._nonce, self.enc_counter
+        )
         return enc
 
     def decrypt(self, msg: bytes) -> bytes:
         """Return decrypted Message."""
         dec = self._dec_cipher.decrypt(msg)
         self._dec_counter += 1
-        self._dec_cipher = get_cipher(self._key, self._nonce, self.dec_counter)
+        self._dec_cipher = get_cipher(
+            self._host_type, self._key, self._nonce, self.dec_counter
+        )
         return dec
 
     @property
@@ -384,23 +418,25 @@ class SessionCipher():
         return self._dec_counter
 
 
-class StreamECDH():
+class StreamECDH:
     """ECDH Container for Stream."""
 
+    @staticmethod
     def get_handshake_key(handshake: bytes = None):
         """Return random key for ECDH."""
         handshake_key = handshake or get_random_bytes(16)
         log_bytes("Handshake Key", handshake_key)
         return handshake_key
 
+    @staticmethod
     def set_local_ec(key: bytes = None):
         """Init Local EC Key object."""
         key = key or get_random_bytes(32)
         private = from_b(key)
-        private_key = ec.derive_private_key(
-            private, ec.SECP256K1(), default_backend())
+        private_key = ec.derive_private_key(private, ec.SECP256K1(), default_backend())
         return private_key
 
+    @staticmethod
     def set_private_key(local_ec):
         """Return Private Key for ECDH."""
         private_numbers = local_ec.private_numbers().private_value
@@ -408,14 +444,15 @@ class StreamECDH():
         log_bytes("Private Key", private_key)
         return private_key
 
+    @staticmethod
     def set_public_key(local_ec):
         """Return Public Key for ECDH."""
         pub = local_ec.public_key()
-        public_key = pub.public_bytes(
-            Encoding.X962, PublicFormat.UncompressedPoint)
+        public_key = pub.public_bytes(Encoding.X962, PublicFormat.UncompressedPoint)
         log_bytes("Public Key", public_key)
         return public_key
 
+    @staticmethod
     def get_key_sig(handshake_key, public_key):
         """Return authenticated signature of public key."""
         hmac = HMAC.new(key=handshake_key, msg=public_key, digestmod=SHA256)
@@ -423,13 +460,15 @@ class StreamECDH():
         log_bytes("Public Key Sig", local_sig)
         return local_sig
 
+    @staticmethod
     def get_secret(local_key, remote_key: bytes):
         """Return derived secret from ECDH exchange."""
         remote_key = ec.EllipticCurvePublicKey.from_encoded_point(
-            ec.SECP256K1(), remote_key)
+            ec.SECP256K1(), remote_key
+        )
         remote_key.public_numbers()
         secret = local_key.exchange(ec.ECDH(), remote_key)
-        log_bytes('Secret', secret)
+        log_bytes("Secret", secret)
         return secret
 
     def __init__(self, handshake: bytes = None, private_key: bytes = None):
