@@ -4,13 +4,13 @@ import time
 import logging
 from collections import deque
 
+import av
 import sounddevice
 from PySide6 import QtCore, QtMultimedia, QtWidgets
 from PySide6.QtCore import Qt  # pylint: disable=no-name-in-module
 from PySide6.QtMultimedia import QAudioDevice
-from pyremoteplay.av import AVReceiver
+from pyremoteplay.receiver import AVReceiver
 from pyremoteplay.device import RPDevice
-from pyremoteplay.feedback import Controller
 
 from .joystick import JoystickWidget
 from .controls import ControlsWidget
@@ -30,18 +30,12 @@ class QtReceiver(AVReceiver):
         self.audio_signal = None
         self.rgb = True
 
-    def handle_video(self, buf):
+    def handle_video(self, frame: av.VideoFrame):
         """Handle video frame."""
-        frame = self.decode_video_frame(buf)
-        if frame is None:
-            return
         self.video_signal.emit(frame)
 
-    def handle_audio(self, buf):
+    def handle_audio(self, frame: av.AudioFrame):
         """Handle Audio Frame."""
-        frame = self.decode_audio_frame(buf)
-        if frame is None:
-            return
         self.audio_signal.emit(frame)
 
     def set_signals(self, video_signal, audio_signal):
@@ -221,7 +215,7 @@ class RPWorker(QtCore.QObject):
             user,
             resolution=options.get("resolution"),
             fps=options.get("fps"),
-            av_receiver=QtReceiver(),
+            receiver=QtReceiver(),
             codec=codec,
             hdr=options.get("hdr"),
             quality=options.get("quality"),
@@ -231,9 +225,9 @@ class RPWorker(QtCore.QObject):
         """Start Session."""
         _LOGGER.debug("Session Start")
         if self.window:
-            self.session.av_receiver.rgb = False if self.window.use_opengl else True
+            self.session.receiver.rgb = False if self.window.use_opengl else True
         if standby:
-            self.session.av_receiver = None
+            self.session.receiver = None
         started = await self.device.connect()
 
         if not started:
@@ -246,9 +240,9 @@ class RPWorker(QtCore.QObject):
             _LOGGER.info("Standby Success: %s", result)
             self.stop(standby=True)
             return
-        self.controller = Controller(self.session)
+        self.controller = self.device.controller
         self.controller.start()
-        self.session.av_receiver.set_signals(
+        self.session.receiver.set_signals(
             self.window.video_frame, self.window.audio_frame
         )
         self.window.audio_frame.connect(
@@ -412,7 +406,7 @@ class StreamWindow(QtWidgets.QWidget):
 
     def _init_audio(self):
         self.audio_thread.start(
-            self.audio_device, self.rp_worker.session.av_receiver.audio_config
+            self.audio_device, self.rp_worker.session.receiver.audio_config
         )
 
     def _init_fps(self):
