@@ -1,8 +1,10 @@
 """Remote Play Session."""
+from __future__ import annotations
 import asyncio
 import logging
 import socket
 import time
+from typing import Union
 from base64 import b64decode, b64encode
 from concurrent.futures import ThreadPoolExecutor
 from enum import IntEnum
@@ -13,6 +15,7 @@ import requests
 from Cryptodome.Random import get_random_bytes
 from pyee import ExecutorEventEmitter
 
+from pyremoteplay.receiver import AVReceiver
 from .av import AVHandler
 from .const import (
     FPS,
@@ -26,6 +29,7 @@ from .const import (
     USER_AGENT,
     Resolution,
     StreamType,
+    Quality,
 )
 from .crypt import SessionCipher
 from .ddp import async_get_status, async_wakeup
@@ -164,7 +168,20 @@ def _gen_did() -> bytes:
 
 
 class Session:
-    """RP Session Async."""
+    """Remote Play Session Async.
+
+    :param host: IP Address of Remote Play Host
+    :param profile: Profile data to connect with. From registering
+    :param receiver: A receiver for handling video and audio frames
+    :param loop: A running asyncio event loop
+    :param resolution: The resolution of video stream.
+        One of '360p', '540p', '720p', '1080p'
+    :param fps: Frames per second for video stream.
+        One of 'low', '30' or 'high', '60'
+    :param quality: Quality of video stream. Name of member of `Quality` enum or `Quality` enum
+    :param codec: Video Codec to use. One of 'h264' or 'hevc'
+    :param hdr: If True use HDR for video. Ignored if codec is 'h264'
+    """
 
     STATE_INIT = "init"
     STATE_READY = "ready"
@@ -222,11 +239,11 @@ class Session:
         self,
         host: str,
         profile: dict,
-        av_receiver=None,
-        loop=None,
+        receiver: AVReceiver = None,
+        loop: asyncio.AbstractEventLoop = None,
         resolution="360p",
-        fps="low",
-        quality="very_low",
+        fps: Union[str, int] = "low",
+        quality: Union[str, Quality] = "very_low",
         codec="h264",
         hdr=False,
         **kwargs,
@@ -251,16 +268,12 @@ class Session:
         self._kwargs = kwargs
         self._hdr = hdr
         self._codec = codec
-        self.quality = quality
         self.max_width = self.max_height = None
-        self.fps = FPS.preset(fps)
-        self.resolution = Resolution.preset(resolution)
         self.error = ""
-        self.av_receiver = av_receiver
+        self._receiver = None
         self.av_handler = AVHandler(self)
         self.events = ExecutorEventEmitter()
-
-        self.loop = asyncio.get_event_loop() if loop is None else loop
+        self.loop = loop
         self._protocol = None
         self._transport = None
         self._tasks = []
@@ -268,8 +281,19 @@ class Session:
 
         self._ready_event = None
         self._stop_event = None
-        self.stream_ready = None
         self.receiver_started = None
+        self.stream_ready = None
+
+        if quality.upper() in Quality.__members__:
+            self.quality = quality
+        elif isinstance(quality, Quality):
+            self.quality = quality.name
+        else:
+            self.quality = Quality.DEFAULT.name
+        self.fps = FPS.preset(fps)
+        self.resolution = Resolution.preset(resolution)
+
+        self.set_receiver(receiver)
 
     def _init_profile_kwargs(self, device: dict) -> bool:
         """Return True if can init profile from kwargs."""
@@ -492,8 +516,10 @@ class Session:
         self._stop_event = asyncio.Event()
         self.receiver_started = asyncio.Event()
         self.stream_ready = asyncio.Event()
-
         _LOGGER.debug("Running Async")
+
+        if not self.loop:
+            self.loop = asyncio.get_running_loop()
         status = await self._check_host()
         if not status[0]:
             self.error = f"Host @ {self._host} is not reachable."
@@ -548,8 +574,8 @@ class Session:
             return
         stop_event = self._stop_event if not test else asyncio.Event()
         cb_stop = self._cb_stop_test if test else None
-        if not test and self.av_receiver:
-            self.av_handler.add_receiver(self.av_receiver)
+        if not test and self.receiver:
+            self.av_handler.add_receiver(self.receiver)
             _LOGGER.info("Waiting for Receiver...")
             self.loop.create_task(self.receiver_started.wait())
         self._stream = RPStream(
@@ -579,7 +605,8 @@ class Session:
         _LOGGER.debug("Session Received Stop Signal")
         if self._stream:
             self._stream.stop()
-        self._stop_event.set()
+        if self._stop_event:
+            self._stop_event.set()
         if self._tasks:
             for task in self._tasks:
                 task.cancel()
@@ -612,6 +639,20 @@ class Session:
         """Run blocking function in executor. Called from sync method."""
         asyncio.ensure_future(self.run_io(func, *args, **kwargs))
 
+    def set_receiver(self, receiver: AVReceiver):
+        """Set AV Receiver. Should be set before starting session."""
+        cls = AVReceiver
+        if receiver is None:
+            return
+        if not isinstance(receiver, AVReceiver):
+            raise ValueError(f"Receiver must be a subclass of {cls}")
+        if receiver.__class__ == AVReceiver:
+            raise ValueError(f"Cannot set receiver of abstract class {cls}")
+        old_receiver = self._receiver
+        self._receiver = receiver
+        if old_receiver:
+            old_receiver.close()
+
     @property
     def host(self) -> str:
         """Return host address."""
@@ -688,3 +729,8 @@ class Session:
         except KeyError:
             stream_type = StreamType["H264"]
         return stream_type
+
+    @property
+    def receiver(self) -> AVReceiver:
+        """Return AV Receiver."""
+        return self._receiver
